<?php
namespace Teknisa\CADSUP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\CADSUP\FloatingReportGenerator\FloatingReportInterface;

class Sup04100FornecedorInativoController implements FloatingReportInterface
{

    protected $cadSupUtil;
    protected $crudService;
    protected $dataSourceName = 'fornecedor_inativo';

    private const LIST_PARAMS = [
            'L_CDFORNECED',
            'L_CDFILIAL'
        ];

    public function __construct($cadSupUtil, $crudService) 
    {

        $this->cadSupUtil          = $cadSupUtil;
        $this->crudService         = $crudService;
        
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) 
    {
        $oldFilterCriteria = $request->getFilterCriteria();

        $item_list = array();
        $params = [ 
            'P_DTINI',
            'P_DTFIN',
            'P_CONSIDERA',
            'L_CDFORNECED',
            'L_CDFILIAL',
        ];

        $result = $this->treatRequestParameters($request);
        $queryParams = $result['queryParams'];
        $NRLISTA_FILIAL = $result['nrListaFilial'];
        $NRLISTA_FORNECEDOR = $result['nrListaFornecedor'];
        
        $page = $oldFilterCriteria->getPage();
        $pageSize = $oldFilterCriteria->getPageSize();
        $filterCriteria = new FilterCriteria($this->dataSourceName, $queryParams, $page, $pageSize);
        
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        
        $filterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $params);
        
        $this->crudService->find($response, $filterCriteria);
        
        $this->cadSupUtil->deleteLista($NRLISTA_FILIAL);

        $this->cadSupUtil->deleteLista($NRLISTA_FORNECEDOR);
        
    }
    
    public function treatRequestParameters(DTO\Request\Filter $request) {
        
        $oldFilterCriteria = $request->getFilterCriteria();
        $filters = $oldFilterCriteria->getConditions();
        
        $queryParams = array();
        
        $NRLISTA_FILIAL = null;
        $NRLISTA_FORNECEDOR = null;
        
        foreach($filters as $filter) {
            if($filter['columnName'] === 'NMFILIAL') {

                $NRLISTA_FILIAL = $this->cadSupUtil->insertLista($filter['value']);
                $queryParams[] = $this->setTemplate('L_CDFILIAL', $NRLISTA_FILIAL);
                
                
            } else if($filter['columnName'] === 'NMFORNECEDOR') {
                
                $NRLISTA_FORNECEDOR = $this->cadSupUtil->insertLista($filter['value']);
                
                $queryParams[] = $this->setTemplate('L_CDFORNECED', $NRLISTA_FORNECEDOR);
                
                
            } else if($filter['columnName'] === 'INTERVAL_DT') {
                
                $queryParams[] = $this->setTemplate('P_DTINI', $filter['value'][0]);
                $queryParams[] = $this->setTemplate('P_DTFIN', $filter['value'][1]);
                
            } else if($filter['columnName'] === 'TIPO') {
                
                $queryParams[] = $this->setTemplate('P_CONSIDERA', $filter['value']);
                
            } else {
                $queryParams[] = ($filter);
            }
        }
        return array (
            "queryParams"=>$queryParams,
            "nrListaFilial"=>$NRLISTA_FILIAL,
            "nrListaFornecedor"=>$NRLISTA_FORNECEDOR
        );
    }
    
    protected function setTemplate($name, $value, $operator = '=') {
        return $currentParameter = array(
            'columnName' => $name,
            'operator'   => $operator,
            'value'      => $value
        );
    }
    
    public function beforeGenerateReport(DTO\Request $request){
        
        if($request instanceof DTO\Request\Filter){
            $parametersToFilter = $this->treatRequestParameters($request);
            $filterQueryParams = $parametersToFilter['queryParams'];
            $this->setProtectedProperty($request->getFilterCriteria(), 'conditions', $filterQueryParams);
        }
    }
    
    public function afterGenerateReport(DTO\Request $request, DTO\Response $response){
        
        if($request instanceof DTO\Request\Filter) {
            $filterConditions = $request->getFilterCriteria()->getConditions();
            
            $paramsList = array_filter($filterConditions, function($currentFilter){
                return array_search($currentFilter['columnName'], self::LIST_PARAMS) !== false;
            }, ARRAY_FILTER_USE_BOTH);

            if(!empty($paramsList)){
                foreach($paramsList as $params){
                    $this->cadSupUtil->deleteLista($params['value']);
                }
            }
        }
    }
    
    private function setProtectedProperty($obj, $property, $value) {
        $reflection = new \ReflectionClass($obj);
        $property = $reflection->getProperty($property);
        $property->setAccessible(true);
        return $property->setValue($obj, $value);
    }
}